@@ -1,36 +1,50 @@
 import pathlib
 from typing import (
-    Any, Collection, Dict, Iterable, NamedTuple, Optional, Set, Tuple,
+    TYPE_CHECKING, Any, Collection, Dict, List, NamedTuple, Optional, Set,
+    Tuple, Union,
 )
 
-# We support these resource types. The order matters because it determines the
-# order in which the manifests will be grouped in the output files.
-DEFAULT_PRIORITIES = (
-    # Namespaces must come first to ensure the other resources can be created
-    # within them.
-    "Namespace",
+if TYPE_CHECKING:
+    from dataclasses import dataclass
+else:
+    import dataclasses
+    from pydantic.dataclasses import dataclass
 
-    # Custom Resources
-    "CustomResourceDefinition",
-
-    # Configuration and PVC before Deployments & friends use them.
-    "ConfigMap", "Secret", "PersistentVolumeClaim",
-
-    # RBAC.
-    "ClusterRole", "ClusterRoleBinding", "Role", "RoleBinding", "ServiceAccount",
-
-    # Define Services before creating Deployments & friends.
-    "Service", "PodDisruptionBudget",
-
-    # Everything that will spawn pods.
-    "CronJob", "Deployment", "DaemonSet", "StatefulSet", "HorizontalPodAutoscaler",
-
-    # Ingresses should be after Deployments & friends.
-    "Ingress",
-)
 
 Filepath = pathlib.Path
 
+# Square will source all its default values from this configuration file.
+DEFAULT_CONFIG_FILE = Filepath(__file__).parent.parent / "resources/defaultconfig.yaml"
+
+# Square will first save/deploy the resources in this list in this order.
+# Afterwards it will move on to all those resources not in this list. The order
+# in which it does that is undefined.
+DEFAULT_PRIORITIES = (
+    # Custom Resources should come first.
+    "CustomResourceDefinition",
+
+    # Commone non-namespaced resources.
+    "ClusterRole", "ClusterRoleBinding",
+
+    # Namespaces must come before any namespaced resources,
+    "Namespace",
+
+    # Configuration and PVC before Deployments & friends use them.
+    "ConfigMap", "PersistentVolumeClaim", "Secret",
+
+    # RBAC.
+    "Role", "RoleBinding", "ServiceAccount",
+
+    # Define Services before creating Deployments & friends.
+    "PodDisruptionBudget", "Service",
+
+    # Everything that will spawn pods.
+    "CronJob", "DaemonSet", "Deployment", "StatefulSet",
+
+    # Other.
+    "HorizontalPodAutoscaler", "Ingress",
+)
+
 
 # -----------------------------------------------------------------------------
 #                                  Kubernetes
@@ -144,20 +158,27 @@ class DeploymentPlanMeta(NamedTuple):
 # -----------------------------------------------------------------------------
 #                             Square Configuration
 # -----------------------------------------------------------------------------
-class Selectors(NamedTuple):
+def _factory(ret):
+    return dataclasses.field(default_factory=lambda: ret)
+
+
+@dataclass
+class Selectors:
     """Comprises all the filters to select manifests."""
-    kinds: Set[str]
-    namespaces: Optional[Iterable[str]]
-    labels: Optional[Set[Tuple[str, str]]]
+    kinds: Set[str] = _factory(set(DEFAULT_PRIORITIES))
+    namespaces: List[str] = _factory([])
+    labels: Set[Tuple[str, str]] = _factory(set())
 
 
-class GroupBy(NamedTuple):
+@dataclass
+class GroupBy:
     """Define how to organise downloaded manifest on the files system."""
-    label: str                  # "app"
-    order: Iterable[str]        # ["ns", "label=app", kind"]
+    label: str = ""                  # "app"
+    order: List[str] = _factory([])  # ["ns", "label=app", kind"]
 
 
-class Config(NamedTuple):
+@dataclass
+class Config:
     """Uniform interface into top level Square API."""
     # Path to local manifests eg "./foo"
     folder: Filepath
@@ -166,18 +187,21 @@ class Config(NamedTuple):
     kubeconfig: Filepath
 
     # Kubernetes context (use `None` to use the default).
-    kube_ctx: Optional[str]
+    kubecontext: Optional[str]
 
     # Only operate on resources that match the selectors.
-    selectors: Selectors = Selectors(kinds=set(DEFAULT_PRIORITIES),
-                                     namespaces=None,
-                                     labels=set())
+    selectors: Selectors = Selectors()
 
     # Sort the manifest in this order, or alphabetically at the end if not in the list.
-    priorities: Collection[str] = tuple(DEFAULT_PRIORITIES)
+    priorities: List[str] = _factory(list(DEFAULT_PRIORITIES))
 
     # How to structure the folder directory when syncing manifests.
-    groupby: GroupBy = GroupBy("", tuple())
+    groupby: GroupBy = GroupBy()
+
+    # Define which fields to skip for which resource.
+    filters: Dict[str, List[Union[str, dict]]] = _factory({})
+
+    version: str = ""
 
 
 # -----------------------------------------------------------------------------
