@@ -1,10 +1,7 @@
 import argparse
 import logging
 import os
-import pathlib
 import re
-import sys
-import textwrap
 from typing import Optional, Tuple
 
 import colorama
@@ -12,7 +9,7 @@ import square
 import square.square
 from square import __version__
 from square.dtypes import (
-    DEFAULT_PRIORITIES, Config, Filepath, GroupBy, Selectors,
+    DEFAULT_CONFIG_FILE, Config, Filepath, GroupBy, Selectors,
 )
 
 # Convenience: global logger instance to avoid repetitive code.
@@ -21,20 +18,8 @@ logit = logging.getLogger("square")
 
 def parse_commandline_args():
     """Return parsed command line."""
-    name = os.path.basename(__file__)
-    description = textwrap.dedent(f'''
-    Manage Kubernetes manifests.
-
-    Examples:
-      {name} get
-      {name} plan
-      {name} apply
-    ''')
-
     def _validate_label(label: str) -> Tuple[str, ...]:
-        """Convert resource `kind` from aliases to canonical name.
-        For instance, `svc` -> `Service`.
-        """
+        """Unpack the labels: "app=square" -> ("app", "square") """
         pat = re.compile(r"^[a-z0-9][-a-z0-9_.]*=[-A-Za-z0-9_.]*[A-Za-z0-9]$")
         if pat.match(label) is None:
             raise argparse.ArgumentTypeError(label)
@@ -43,49 +28,59 @@ def parse_commandline_args():
     # A dummy top level parser that will become the parent for all sub-parsers
     # to share all its arguments.
     parent = argparse.ArgumentParser(
-        description=description,
         formatter_class=argparse.RawDescriptionHelpFormatter,
-        add_help=False
+        add_help=False,
+        prog="square",
     )
     parent.add_argument(
         "-v", "--verbosity", action="count", default=0,
         help="Log level (-v: WARNING -vv: INFO -vvv: DEBUG)"
     )
     parent.add_argument(
-        "-n", "--namespace", type=str, nargs="*",
-        metavar="ns", dest="namespaces",
-        help="List of namespaces (omit to consider all)",
+        "--info", action='store_true',
+        help="Print compiled configuration (useful to debug)"
     )
     parent.add_argument(
-        "-l", "--labels", type=_validate_label, nargs="*",
-        metavar="labels", dest="labels", default=tuple(),
-        help="Only select resources with these labels (eg 'app=foo')",
+        "-c", "--config", type=str, default="", dest="configfile",
+        help="Read configuration from this file"
+    )
+    parent.add_argument(
+        "--no-config", dest="no_config", action="store_true",
+        help="Ignore .square.yaml file"
+    )
+    parent.add_argument(
+        "-n", "--namespace", "--namespaces", type=str, nargs="*",
+        metavar="ns", dest="namespaces", default=None,
+        help="List of namespaces (omit to select all)",
+    )
+    parent.add_argument(
+        "-l", "--label", "--labels", type=_validate_label, nargs="*",
+        metavar="labels", dest="labels", default=None,
+        help="Target only these labels (eg 'app=foo')",
     )
     parent.add_argument(
         "-p", "--priorities", nargs="*",
-        metavar="priorities", dest="priorities", default=tuple(DEFAULT_PRIORITIES),
+        metavar="priorities", dest="priorities", default=None,
         help="Sort resource in this order when saving and applying",
     )
     parent.add_argument(
         "--kubeconfig", type=str, metavar="path",
-        default=os.environ.get("KUBECONFIG", None),
-        help="Location of kubeconfig file (defaults to env KUBECONFIG)",
+        default=None, help="Location of kubeconfig file",
     )
     parent.add_argument(
-        "--folder", type=str, metavar="path",
-        default=os.environ.get("SQUARE_FOLDER", "./"),
-        help="Manifest folder (defaults to env SQUARE_FOLDER first then ./)",
-    )
-    parent.add_argument(
-        "--context", type=str, metavar="ctx", dest="ctx", default=None,
+        "--kubecontext", type=str, metavar="kubecontext", default=None,
         help="Kubernetes context (defaults to default context)",
     )
+    parent.add_argument(
+        "--folder", type=str, metavar="path", default=None,
+        help="Manifest folder (defaults to ./)",
+    )
 
     # The primary parser for the top level options (eg GET, PATCH, ...).
-    parser = argparse.ArgumentParser(add_help=True)
+    parser = argparse.ArgumentParser(add_help=True, prog="square")
     subparsers = parser.add_subparsers(
         help='Mode', dest='parser', metavar="ACTION",
-        title="Operation", required=True
+        title="Operation", required=True,
     )
 
     # Configuration for `kinds` positional arguments. Every sub-parser must
@@ -94,7 +89,7 @@ def parse_commandline_args():
     kinds_kwargs = {
         "dest": "kinds",
         "type": str,
-        "nargs": '+',
+        "nargs": '*',
         "metavar": "resource",
     }
 
@@ -126,9 +121,20 @@ def parse_commandline_args():
         'version', help="Show Square version and exit", parents=[parent]
     )
 
+    # Sub-command CONFIG.
+    subparsers.add_parser(
+        'config', help="Create .square.yaml (works with --folder)", parents=[parent]
+    )
+
     # Parse the actual arguments.
     param = parser.parse_args()
 
+    # Set `param.kinds` to `None` if the user did not specify any. This works
+    # around a argparse idiosyncrasy where one cannot specify a `None` default
+    # value for an optional list of positional arguments.
+    kinds = getattr(param, "kinds", None)
+    param.kinds = kinds if kinds else None
+
     return param
 
 
@@ -164,78 +170,131 @@ def compile_config(cmdline_param) -> Tuple[Config, bool]:
     err_resp = Config(
         folder=Filepath(""),
         kubeconfig=Filepath(""),
-        kube_ctx=None,
-        selectors=Selectors(set(), None, None),
-        groupby=GroupBy("", tuple()),
-        priorities=tuple(),
+        kubecontext=None,
+        selectors=Selectors(set(), namespaces=[], labels=set()),
+        groupby=GroupBy("", []),
+        priorities=[],
     ), True
 
     # Convenience.
     p = cmdline_param
 
+    # Load the default configuration unless the user specified an explicit one.
+    if p.configfile:
+        logit.info(f"Loading configuration file <{p.configfile}>")
+        cfg, err = square.square.load_config(p.configfile)
+
+        # Use the folder the `load_config` function determined.
+        folder = cfg.folder
+
+        # Look for `--kubeconfig`. Default to the value in config file.
+        kubeconfig = p.kubeconfig if p.kubeconfig else cfg.kubeconfig
+    else:
+        # Pick the configuration file, depending on whether the user specified
+        # `--no-config`, `--config` and whether a `.square.yaml` file exists.
+        default_cfg = DEFAULT_CONFIG_FILE
+        dot_square = Filepath(".square.yaml")
+        if p.no_config:
+            cfg_file = default_cfg
+        else:
+            cfg_file = dot_square if dot_square.exists() else default_cfg
+        logit.info(f"Loading configuration file <{cfg_file}>")
+        cfg, err = square.square.load_config(cfg_file)
+        del dot_square, default_cfg
+
+        # Use the current working directory as the folder directory because the
+        # user did not specify an explicit configuration file which would
+        # contain the desired folder.
+        folder = Filepath.cwd()
+
+        # Look for `--kubeconfig`, fall back to KUBECONFIG env var.
+        kubeconfig = p.kubeconfig if p.kubeconfig else os.getenv("KUBECONFIG", None)
+
+        # Abort if neither `--kubeconfig` nor KUBECONFIG env var.
+        if kubeconfig is None:
+            logit.error("Must specify a Kubernetes config file.")
+            return err_resp
+    if err:
+        return err_resp
+
+    # Override the folder if user specified "--folder".
+    folder = folder if p.folder is None else p.folder
+
+    # ------------------------------------------------------------------------
+    # GroupBy (determines the folder hierarchy that GET will create).
+    # In : `--groupby ns kind label=app`
+    # Out: GroupBy(order=["ns", "kind", "label"], label="app")
+    # ------------------------------------------------------------------------
+    # Unpack the ordering and replace all `label=*` with `label`.
+    # NOTE: `p.groups` does not necessarily exist because the option only makes
+    #        sense for eg `square GET` and is thus not implemented for `square apply`.
+    order = getattr(p, "groupby", None)
+    if order is None:
+        groupby = cfg.groupby
+    else:
+        clean_order = [_ if not _.startswith("label") else "label" for _ in order]
+        if not set(clean_order).issubset({"ns", "kind", "label"}):
+            logit.error(f"Invalid definition of `groupby`")
+            return err_resp
+
+        labels = [_ for _ in order if _.startswith("label")]
+        if len(labels) > 1:
+            logit.error("Can only specify one `label=<name>` in `--groupby`.")
+            return err_resp
+
+        # Unpack the label name from the `--groupby` argument.
+        # Example, if user specified `--groupby label=app` then we need to extract
+        # the `app` part. This also includes basic sanity checks.
+        label_name = ""
+        if len(labels) == 1:
+            try:
+                _, label_name = labels[0].split("=")
+                assert len(label_name) > 0
+            except (ValueError, AssertionError):
+                logit.error(f"Invalid label specification <{labels[0]}>")
+                return err_resp
+
+        # Compile the final `GroupBy` structure.
+        groupby = GroupBy(order=list(clean_order), label=label_name)
+        del order, clean_order, label_name
+
+    # ------------------------------------------------------------------------
+    # General: folder, kubeconfig, kubecontext, ...
+    # ------------------------------------------------------------------------
+    kinds = set(p.kinds) if p.kinds else cfg.selectors.kinds
+
+    # Use the value from the (default) config file unless the user overrode
+    # them on the command line.
+    kubeconfig = Filepath(kubeconfig)
+    kubecontext = p.kubecontext or cfg.kubecontext
+    namespaces = cfg.selectors.namespaces if p.namespaces is None else p.namespaces
+    sel_labels = cfg.selectors.labels if p.labels is None else set(p.labels)
+    priorities = p.priorities or cfg.priorities
+    selectors = Selectors(kinds, namespaces, sel_labels)
+
+    # Use filters from (default) config file because they cannot be specified
+    # on the command line.
+    filters = cfg.filters
+
+    # ------------------------------------------------------------------------
+    # Verify inputs.
+    # ------------------------------------------------------------------------
     # Abort without credentials.
-    kubeconfig = Filepath(p.kubeconfig)
     if not kubeconfig.exists():
         logit.error(f"Cannot find Kubernetes config file <{kubeconfig}>")
         return err_resp
 
-    # Remove duplicates but retain the original order of "p.kinds". This is
-    # a "trick" that will only work in Python 3.7+ which guarantees a stable
-    # insertion order for dictionaries (but not sets).
-    p.kinds = set(dict.fromkeys(p.kinds))
-
-    # Expand the "all" resource (if present) and ignore all other resources, if
-    # any were specified.
-    if "all" in p.kinds:
-        p.kinds.clear()
-
-    # Folder must be `Path` object.
-    folder = pathlib.Path(p.folder)
-
-    # Specify the selectors (see definition of `dtypes.Selectors`).
-    selectors = Selectors(p.kinds, p.namespaces, set(p.labels))
-
-    # ------------------------------------------------------------------------
-    # Unpack the folder hierarchy. For example:
-    # From: `--groupby ns kind label=app` ->
-    # To  : GroupBy(order=["ns", "kind", "label"], label="app")
-    # ------------------------------------------------------------------------
-    # Unpack the ordering and replace all `label=*` with `label`.
-    order = getattr(p, "groupby", None) or []
-    clean_order = [_ if not _.startswith("label") else "label" for _ in order]
-    if not set(clean_order).issubset({"ns", "kind", "label"}):
-        logit.error(f"Invalid resource names in <{order}>")
-        return err_resp
-
-    labels = [_ for _ in order if _.startswith("label")]
-    if len(labels) > 1:
-        logit.error("Can only specify one label in file hierarchy")
-        return err_resp
-
-    # Unpack the label name if the user specified it as part of `--groupby`.
-    # Example, if user specified `--groupby label=app` then we need to extract
-    # the `app` part. This also includes basic sanity checks.
-    label_name = ""
-    if len(labels) == 1:
-        try:
-            _, label_name = labels[0].split("=")
-            assert len(label_name) > 0
-        except (ValueError, AssertionError):
-            logit.error(f"Invalid label specification <{labels[0]}>")
-            return err_resp
-    groupby = GroupBy(order=tuple(clean_order), label=label_name)
-    del order, clean_order, label_name
-
     # -------------------------------------------------------------------------
     # Assemble the full configuration and return it.
     # -------------------------------------------------------------------------
     cfg = Config(
         folder=folder,
         kubeconfig=kubeconfig,
-        kube_ctx=p.ctx,
+        kubecontext=kubecontext,
         selectors=selectors,
         groupby=groupby,
-        priorities=p.priorities,
+        priorities=priorities,
+        filters=filters,
     )
     return cfg, False
 
@@ -285,14 +344,10 @@ def apply_plan(cfg: Config, confirm_string: Optional[str]) -> bool:
 def sanitise_resource_kinds(cfg: Config) -> Tuple[Config, bool]:
     """Populate the `Selector.kinds` with those the user specified on the commandline."""
     # Create a K8sConfig instance because it will contain all the info we need.
-    k8sconfig, err = square.k8s.cluster_config(cfg.kubeconfig, cfg.kube_ctx)
+    k8sconfig, err = square.k8s.cluster_config(cfg.kubeconfig, cfg.kubecontext)
     if err:
         return (cfg, True)
 
-    # If the user did not specify any selectors then we assume he wants all resources.
-    if len(cfg.selectors.kinds) == 0:
-        cfg.selectors.kinds.update(k8sconfig.kinds)
-
     # Translate colloquial names to their canonical counterpart if possible.
     # Example: "svc" -> "Service"
     # Do nothing if we do not recognise the name. This can happen because the
@@ -307,12 +362,36 @@ def sanitise_resource_kinds(cfg: Config) -> Tuple[Config, bool]:
     return (cfg, False)
 
 
+def show_info(cfg: Config) -> bool:
+    """Show some info about the configuration that will be used."""
+    sel = cfg.selectors
+
+    print()
+    print(f"Kubeconfig : {cfg.kubeconfig} (Context={cfg.kubecontext})")
+    print("Manifests  :", cfg.folder)
+    print("Namespaces :", sel.namespaces if sel.namespaces else "all")
+    print("Kinds      :", sel.kinds)
+    print("Labels     :", sel.labels)
+    return False
+
+
 def main() -> int:
     param = parse_commandline_args()
     if param.parser == "version":
         print(__version__)
         return 0
 
+    if param.parser == "config":
+        fname = Filepath(param.folder) / ".square.yaml"
+        fname.parent.mkdir(parents=True, exist_ok=True)
+        fname.write_text(DEFAULT_CONFIG_FILE.read_text())
+        print(
+            f"Created configuration file <{fname}>.\n"
+            "Please open the file in an editor and adjust the values, most notably "
+            "<kubeconfig>, <kubecontext>, <folder> and <selectors.labels>."
+        )
+        return 0
+
     # Initialise logging.
     square.square.setup_logging(param.verbosity)
 
@@ -323,7 +402,9 @@ def main() -> int:
         return 1
 
     # Do what the user asked us to do.
-    if param.parser == "get":
+    if param.info:
+        err = show_info(cfg)
+    elif param.parser == "get":
         err = square.square.get_resources(cfg)
     elif param.parser == "plan":
         plan, err = square.square.make_plan(cfg)
@@ -336,7 +417,3 @@ def main() -> int:
 
     # Return error code.
     return 1 if err else 0
-
-
-if __name__ == '__main__':
-    sys.exit(main())
