@@ -99,7 +99,7 @@ def select(manifest: dict, selectors: Selectors) -> bool:
 
     # Include the resource if it is a) namespaced b) we have namespace
     # selectors and c) the resource matches one of them.
-    if ns and selectors.namespaces is not None and ns not in selectors.namespaces:
+    if ns and selectors.namespaces and ns not in selectors.namespaces:
         logit.debug(f"Namespace {ns} does not match selector {selectors.namespaces}")
         return False
 
@@ -108,8 +108,8 @@ def select(manifest: dict, selectors: Selectors) -> bool:
     # labels or not.
     labels = {(k, v) for k, v in labels.items()}
 
-    # Unless the label selector is None, the resource must match it.
-    if selectors.labels is not None:
+    # The resource must match the label selectors, if we have any such selectors.
+    if selectors.labels:
         if not selectors.labels.issubset(labels):
             logit.debug(f"Labels {labels} do not match selector {selectors.labels}")
             return False
@@ -245,7 +245,7 @@ def unpack(manifests: LocalManifestLists) -> Tuple[ServerManifests, bool]:
         manifests: LocalManifestLists
 
     Returns:
-        ServerManifests: flattened version of `data`.
+        ServerManifests: flattened version of `manifests`.
 
     """
     # Compile a dict that shows which meta manifest was defined in which file.
@@ -436,8 +436,8 @@ def diff(config: Config,
     """
     # Clean up the input manifests because we do not want to diff, for instance,
     # the `status` fields.
-    (srv, _), err1 = strip(k8sconfig, server, square.schemas.EXCLUSION_SCHEMA)
-    (loc, _), err2 = strip(k8sconfig, local, square.schemas.EXCLUSION_SCHEMA)
+    srv, _, err1 = strip(k8sconfig, server, config.filters)
+    loc, _, err2 = strip(k8sconfig, local, config.filters)
     if err1 or err2:
         return ("", True)
 
@@ -454,26 +454,24 @@ def diff(config: Config,
 
 
 def strip(
-        k8sconfig: K8sConfig,
-        manifest: dict,
-        exclusion_schema: Dict[str, dict],
-        ) -> Tuple[Tuple[DotDict, dict], bool]:
-    """Strip `manifest` according to the exclusion filters in `square.schemas`.
+    k8sconfig: K8sConfig,
+    manifest: dict,
+    manifest_filters: Dict[str, list],
+) -> Tuple[DotDict, dict, bool]:
+    """Strip `manifest` according to the filters in `square.schemas`.
 
     Inputs:
         k8sconfig: K8sConfig
         manifest: dict
-        exclusion_schema: Dict[str, dict]
-            fixme: docu
+        manifest_filters: Dict[str, list]
+            See tests for examples
 
     Returns:
-        dict: the removed keys.
+        dict, dict: (stripped manifest, removed keys).
 
     """
-    assert k8sconfig.version is not None
-
     # Convenience: default return value if an error occurs.
-    ret_err: Tuple[Tuple[DotDict, dict], bool] = ((square.dotdict.make({}), {}), True)
+    ret_err: Tuple[DotDict, dict, bool] = (square.dotdict.make({}), {}, True)
 
     # Parse the manifest.
     try:
@@ -486,51 +484,62 @@ def strip(
     except AssertionError:
         return ret_err
 
-    def _update(exclude: dict, manifest: dict):
-        """Recursively traverse the `manifest` and prune it according to `exclude`.
+    def _update(filters: list, manifest: dict):
+        """Recursively traverse the `manifest` and prune it according to `filters`.
 
         Returns dict with the excluded keys.
 
         Raise `KeyError` if an invalid key was found.
 
         """
-        # Iterate over the manifest. Prune all keys that match the exclusions
-        # schema and record them in `removed`.
+        # Split the list of strings and dicts into a dedicated set of string
+        # and dedicated list of dicts.
+        # Example: ["foo", "bar", {"a": "b", "c": "d"}] will become
+        #   {"foo", "bar"} and {"a": "b", "c", "d"}.
+        filter_str = {_ for _ in filters if isinstance(_, str)}
+        filter_map = [_ for _ in filters if isinstance(_, dict)]
+        filter_map = {k: v for d in filter_map for k, v in d.items()}
+
+        # Iterate over the manifest. Prune all keys that match the `filters`
+        # and record them in `removed`.
         removed = {}
         for k, v in list(manifest.items()):
-            # Keep this manifest key because there is no exclusion filter for it.
-            if k not in exclude:
-                continue
-
-            if isinstance(exclude[k], dict):
-                # Recurse into the next dictionary and collect the removed keys.
-                removed[k] = _update(exclude[k], manifest[k])
-            else:
-                # Remove the current key from the manifest.
-                # NOTE: the value of `exclude[k]` is irrelevant. All that
-                # matters is that it is a bool.
+            if k in filter_str:
+                # Remove the entire key (and all sub-fields if present).
+                # NOTE: it does not matter if the key also exists in
+                # `filter_map` - we remove the entire key.
+                logit.debug(f"Remove <{k}>")
                 removed[k] = manifest.pop(k)
+            elif isinstance(v, list) and k in filter_map:
+                # Recursively filter each list element.
+                tmp = [_update(filter_map[k], _) for _ in v]
+                removed[k] = [_ for _ in tmp if _]
 
-            # Remove the key altogether if it has become an empty dict.
-            if manifest.get(k, None) == {}:
+                # Do not leave empty elements in the list.
+                manifest[k] = [_ for _ in v if _]
+            elif isinstance(v, dict) and k in filter_map:
+                # Recursively filter each dictionary element.
+                logit.debug(f"Dive into <{k}>")
+                removed[k] = _update(filter_map[k], manifest[k])
+            else:
+                logit.debug(f"Skip <{k}>")
+
+            # Remove the key from the manifest altogether if it has become empty.
+            if not manifest.get(k, "non-empty"):
                 del manifest[k]
 
         # Remove all empty sub-dictionaries from `removed`.
         return {k: v for k, v in removed.items() if v != {}}
 
-    # Add a default exclusion scheme for the current resource if none exists yet.
-    if manifest["kind"] not in exclusion_schema:
-        logit.warning(f"No exclusion schema for: <{meta.kind}> - using default")
-        exclusion_schema[manifest["kind"]] = {}
-        square.schemas.populate_schemas(exclusion_schema)
+    # Get the filters for the current resource. Use the default one if none exists.
+    filters = manifest_filters.get(manifest["kind"], square.schemas.default())
+    if not square.schemas.valid(filters):
+        return ret_err
 
-    # Get the exclusion scheme for the current resource.
-    exclude = exclusion_schema[manifest["kind"]]
-
-    # Strip down the manifest to its essential parts and return it.
+    # Remove the keys from the `manifest` according to `filters`.
     manifest = copy.deepcopy(manifest)
-    removed = _update(exclude, manifest)
-    return ((square.dotdict.make(manifest), removed), False)
+    removed = _update(filters, manifest)
+    return (square.dotdict.make(manifest), removed, False)
 
 
 def align_serviceaccount(
@@ -773,7 +782,8 @@ def load(folder: Filepath, selectors: Selectors) -> Tuple[
 
     # Compile the list of all YAML files in `folder` but only store their path
     # relative to `folder`.
-    fnames = [_.relative_to(folder) for _ in folder.rglob("*.yaml")]
+    fnames = [(_.relative_to(folder), _.name) for _ in folder.rglob("*.yaml")]
+    fnames = [path for path, name in fnames if not name.startswith(".")]
 
     try:
         # Load the files and abort on error.
@@ -865,10 +875,9 @@ def save(folder: Filepath, manifests: LocalManifestLists,
         None
 
     """
-    # Sort the manifest in each file by priority.
-    out, err = sort_manifests(manifests, priority)
-    if err:
-        return True
+    # Sort the manifest in each file by priority. Ignore the error flag because
+    # `sort_manifests` always succeeds.
+    out, _ = sort_manifests(manifests, priority)
 
     # Ignore all files without manifests, ie empty files.
     out_nonempty = {k: v for k, v in out.items() if len(v) > 0}
@@ -879,6 +888,9 @@ def save(folder: Filepath, manifests: LocalManifestLists,
     out_clean = {k: square.dotdict.undo(v) for k, v in out_nonempty.items()}
     del out_nonempty
 
+    # Ignore all hidden files.
+    out_clean = {k: v for k, v in out_clean.items() if not k.name.startswith(".")}
+
     # Convert all manifest dicts into YAML strings.
     out_final: Dict[Filepath, str] = {}
     try:
@@ -915,7 +927,7 @@ def download(config: Config, k8sconfig: K8sConfig) -> Tuple[ServerManifests, boo
 
     # Ensure `namespaces` is always a list to avoid special casing below.
     all_namespaces: Iterable[Optional[str]]
-    if config.selectors.namespaces is None:
+    if not config.selectors.namespaces:
         all_namespaces = [None]
     else:
         all_namespaces = config.selectors.namespaces
@@ -942,18 +954,18 @@ def download(config: Config, k8sconfig: K8sConfig) -> Tuple[ServerManifests, boo
                 assert not err and manifests is not None
 
                 # Drop all manifest fields except "apiVersion", "metadata" and "spec".
-                ret = {k: strip(k8sconfig, man, square.schemas.EXCLUSION_SCHEMA)
+                ret = {k: strip(k8sconfig, man, config.filters)
                        for k, man in manifests.items()}
 
                 # Ensure `strip` worked for every manifest.
-                err = any((v[1] for v in ret.values()))
+                err = any((v[2] for v in ret.values()))
                 assert not err
 
                 # Unpack the stripped manifests from the `strip` response.
                 # The "if v[0] is not None" is to satisfy MyPy - we already
                 # know they are not None or otherwise the previous assert would
                 # have failed.
-                manifests = {k: v[0][0] for k, v in ret.items() if v[0][0] is not None}
+                manifests = {k: v[0] for k, v in ret.items() if v[0] is not None}
             except AssertionError:
                 # Return nothing, even if we had downloaded other kinds already.
                 return ({}, True)
